import '../style/visual.less';

/**
 * 引入echarts
 * 可以全量引入 import * as echarts from 'echarts';
 * 也可以按需引入 以下代码为按需引入
 */

import * as echarts from 'echarts/core';
import { TooltipComponent, GridComponent } from 'echarts/components';
import { BarChart } from 'echarts/charts';
import { LineChart  } from 'echarts/charts';
import { CanvasRenderer } from 'echarts/renderers';

echarts.use([TooltipComponent, GridComponent, BarChart,LineChart , CanvasRenderer]);


/**
 * 一个很好用的前端查询组件
 */
import jslinq from "jslinq";

/**
 * 引用外置库js对象
 */
// @ts-ignore
import AMap from 'AMap';

/**
 * 数据接口
 * 可以根据业务扩展字段,属于自定义类型, 根据自己实际业务填充字段
 * 这里将类型设置为any, 是为了后面可以做更多的扩展,如果类型简单
 */
interface IRenderConfig {
  xAxis: string[];
  values: number[];
  isMock: boolean;
}

export default class Visual extends WynVisual {

  /**
   * 设置默认数据
   * @private
   */
  private static defaultConfig: IRenderConfig = {
    xAxis: ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'],
    values: [120, 200, 150, 80, 70, 110, 130],
    isMock: true,
  }

  /**
   * 容器中的第一个div,填充数据的空间
   * @private
   */
  private dom: HTMLDivElement;

  /**
   * div中再增嵌套一个div,可以根据后期业务增加多个,调整css即可
   * @private
   */
  private container: HTMLDivElement;


  /**
   * echarts对象
   * @private
   */
  private echartsInstance: echarts.ECharts;

  /**
   * 加载配置项,将所有绘制后的数据存储,以便于渲染图表使用
   * @private
   */
  private renderConfig: IRenderConfig;

  /**
   * 图表配置项, 包含所有图表配置属性项目
   * @private
   */
  private styleConfig: any;


  /**
   * 工具提示
   * @private
   */
  private tooltipFields: any;


  /**
   * 数据格式化
   * @private
   */
  private format: any;
  private displayUnit: any;
  private unitType: any;

  /**
   * 数据钻取联动
   * @private
   */
  private host: any;
  private selectionManager: any;
  private selectionOption: any;
  private items: any;


  /**
   * 数据编订中的属性值名称
   * 后面可能用到
   */
  private numeric_key:any;
  private category_key:any;
  /**
   * 首次加载方法,后期不执行
   * @param dom
   * @param host
   * @param options
   */
  constructor(dom: HTMLDivElement, host: VisualNS.VisualHost, options: VisualNS.IVisualUpdateOptions) {
    super(dom, host, options);

    const img= host.assetsManager.getImage('bg');
    console.log(img);

    const amapToken = host.configurationManager.get('高德地图API');

    console.log(amapToken)

    this.dom=dom;

    this.container = document.createElement('div');
    this.container.id = 'container';

    this.dom.append(this.container);

    this.echartsInstance = echarts.init(dom);

    this.host = host;



    this.selectionManager = host.selectionService.createSelectionManager();
    this.selectionOption = [];
    this.tooltipFields = [];

    this.bindEvents();
  }

  /**
   * 所有项目改变,都会进入update方法
   * @param options
   */
  public update(options: VisualNS.IVisualUpdateOptions) {

    this.styleConfig = options.properties;
    this.items=[];
    /**
     * 获取当前绑定数据
     */
    const plainDataView = options.dataViews[0] && options.dataViews[0].plain;
    /**
     * 判断是否绑定了数据
     */
    if (plainDataView) {
      this.numeric_key = plainDataView.profile.numeric.values[0].display;
      this.category_key = plainDataView.profile.category.values[0].display;


      //排序
      const sortFlage = plainDataView.sort[this.category_key].order;
      const datas = []
      sortFlage.map((serise, index) => {
        datas[index] = plainDataView.data.filter((item) => item[this.category_key] === serise && item[this.numeric_key])
      })
      //显示数据组装
      const values = [];
      const series = [];
      datas.forEach((plain) => {
        const rNum=this.number_formatting(plain[0][this.numeric_key],plain[0][this.category_key]);
        values.push(rNum);
        series.push(plain[0][this.category_key]);
      });
      //数据格式化基础属性获取
      this.format = plainDataView.profile.numeric.options.valueFormat;
      this.displayUnit = plainDataView.profile.numeric.options.valueDisplayUnit;
      this.unitType=plainDataView.profile.numeric.options.unitType;


      //数据联动items组合
      const items = plainDataView.data.reduce((result: any, item: any, i: number) => {
        const selectionId = this.host.selectionService.createSelectionId();
        selectionId.withDimension(plainDataView.profile.category.values[0], item);
        result.push({
          name: item[this.category_key],
          selectionId,
        });
        return result;
      }, []);
      this.items = items;



      this.renderConfig = {
        xAxis: series,
        values,
        isMock: false,
      };


    }else {
      this.renderConfig = Visual.defaultConfig;
    }
    this.render();


    if (this.renderConfig.isMock) {
      this.dom.style.opacity = '0.3';
    } else {
      this.dom.style.opacity = '1';
    }
  }

  /**
   * 需要给图表绑定的事件
   */
  public bindEvents(){
    //鼠标左键
    this.echartsInstance.on('click', (params) => {
      this.host.contextMenuService.hide();
      params.event.event.stopPropagation();
       if (params.event.event["button"] == 0) {
         const sid = this.getNodeSelectionId(this.items[params.dataIndex].name);
         this.selectionManager.clear();
         this.selectionManager.select(sid, true);
       }
    })



    //右键显示
    this.echartsInstance.on('mouseup', (params) => {
      document.oncontextmenu = function () { return false; };
      if (params.event.event["button"] === 2) {
        this.host.contextMenuService.show({
          position: {
            x: params.event.event["offsetX"],
            y: params.event.event["offsetY"],
          },
          menu: true
        }, 10)
        return;
      } else {
        this.host.contextMenuService.hide();
      }
    })
  }

  /**
   * 数据格式化
   * @param number
   * @param format
   * @param displayUnit_3
   */
  public formatData = (number) => {
    const formatService = this.host.formatService;
    let realDisplayUnit = this.displayUnit == null ? "auto" : this.displayUnit;
    if (formatService.isAutoDisplayUnit(realDisplayUnit)) {
      realDisplayUnit = formatService.getAutoDisplayUnit([number]);
    }
    return formatService.format(this.format, number, realDisplayUnit)+(this.unitType?this.unitType:"");
  }

  /**
   * 数据格式化,根据自定义设置中的配置项组装数据
   * @param num
   * @private
   */
  private number_formatting(num,cate) {
    const style = this.styleConfig;
    let ret_number:any;
    style.custom_settings.forEach(x => {
      const _key = x.cate;
      if (_key===cate){
        ret_number={
          value: num,
          itemStyle: {
            color: x.bar_color
          }
        };
      }
    });
    if (ret_number){
      return ret_number;
    }
    return num;
  }

  /**
   * 业务处理,将数据展示在组件上
   */
  public render() {
    const config = this.renderConfig;
    const styleConfig = this.styleConfig;
    const _this=this;
    const options = {
      tooltip: {
        show: styleConfig.tooltip,
        trigger: 'axis',
        backgroundColor: styleConfig.font_color,
        axisPointer: {
          type: 'shadow'
        },
        formatter:function (x){
          let num;
          if (x[0].data.value){
            num= _this.formatData(x[0].data.value);
          }else{
            num= _this.formatData(x[0].data);
          }
          return `${x[0].name}<br/>${_this.numeric_key}:${num}`;
        }
      },
      xAxis: {
        data: config.xAxis
      },
      grid: {
        left: '3%',
        right: '4%',
        bottom: '3%',
        containLabel: true
      },
      yAxis: {},
      series: [{
        type: 'bar',
        data: config.values,
        label: {
          position: "top",
          color:'#fff',
          show: true,
          formatter:function (x){
            if (x.data.value){
              return _this.formatData(x.data.value);
            }else{
              return _this.formatData(x.data);
            }
          }
        }
      }],
      color: styleConfig.palette,
      textStyle: {
        ...styleConfig.textStyle,
        fontSize: parseInt(styleConfig.textStyle.fontSize),
      },
      animation: styleConfig.enableAnimation,
    };

    this.echartsInstance.setOption(options);

    this.echartsInstance.on('click', function (params) {
        _this.clickHandler(params.name);
    });

  }

  /**
   * 点击事件
   * 根据点击的列值,判定当前分类关联的是选中还是取消选中
   * @param key
   */
  private clickHandler = (key: any) => {
    const sid=this.getNodeSelectionId(key);
    if (this.selectionManager.contains(sid)) {
      this.selectionManager.clear(sid)
    }else{
      this.selectionManager.select(sid,true)
    }
  }

  /**
   * 根据分类信息获取选中值
   * @param key
   */
  public getNodeSelectionId = (key: any) => {
    const {selectionId} = this.items.find((item: any) => item.name == key);
    return selectionId
  }
  /**
   * 卸载时将调用
   */
  public onDestroy() {
    this.echartsInstance.dispose();
  }

  /**
   * 改变图表大小后触发
   */
  public onResize() {
    this.echartsInstance.resize();
  }

  /**
   * 返回需要隐藏的属性名称集合
   * @param options
   */
  public getInspectorHiddenState(options: VisualNS.IVisualUpdateOptions): string[] {
    let properties = options.properties;
    let states = [];
    if (!properties.tooltip){
      states=states.concat(["font_color"]);
    }


    return states;
  }

  /**
   * 返回需要显示的属性名称集合
   * @param options
   */
  public getActionBarHiddenState(options: VisualNS.IVisualUpdateOptions): string[] {
    let properties = options.properties;
    let states = [];

    if (properties.tooltip){
      states=states.concat(["font_color"]);
    }
    return states;
  }

  public getColorAssignmentConfigMapping(dataViews: VisualNS.IDataView[]): VisualNS.IColorAssignmentConfigMapping {
    if (!dataViews.length) {
      return null;
    }
    const plain = dataViews[0].plain;
    const colorProfile = plain.profile.numeric.values[0];
    const dimProfile = plain.profile.category.values[0];
    if (!colorProfile || !dimProfile) {
      return null;
    }
    return null;
  }
}